/*
 * HTTPARSER.H
 * Contains templates and parser fuctions.
 *
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __HTTPARSER_H__
#define __HTTPARSER_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <regex.h>
#include <getopt.h>

#define VERSION "v0.5"
#define AUTHOR "Acri Emanuele (crossbower@gmail.com)"

#define MIN_ARG 256
#define MAX_ARG 1024

#define MAX_DATA 2048


/*
 * IP range structure
 */
struct iprange {
    char range[MIN_ARG];    // ip range
    char block[MIN_ARG];    // block
    char addr[MIN_ARG];     // address
    
    int start;     // fist host
    int end;       // last host
    int curr;      // current host
    
    int port;      // port
    int time;      // child lifetime
    
    int ipver;     // ip version
    
    int verbose:1;   // verbose
    int cookiejar:1; // cookie-jar output
};
#define VER_IPV4 4
#define VER_IPV6 6


/*
 * Regular expression used by
 * parseopt() and parseipr() functions
 */
char *reg_ipv4_range =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}[-][0-9]{1,3}$";

char *reg_ipv4_addr =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}$";

char *reg_ipv6_range =
    "^[0-9a-fA-F:]{1,42}[-][0-9a-fA-F]{1,4}$";

char *reg_ipv6_addr =
    "^[0-9a-fA-F:]{1,42}$";

char *reg_number =
    "\\d+?";


/*
 * Http 1.1 request
 */
char http_request[MAX_DATA] =
"GET / HTTP/1.1\r\n" \
"Host: localhost\r\n" \
"User-Agent: Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.3)\r\n" \
"Accept: text/html\r\n" \
"Connection: close\r\n" \
"\r\n";

/*
 * Program usage
 */
void usage(FILE *out, char *error)
{
    char usage[] =
        "\nHTTSquash scanner " VERSION " - " AUTHOR "\n" \
        "Usage:\n" \
        "  httsquash -r range [options]\n" \
        "Options:\n" \
        "  -r\trange of ipv4 or ipv6 addresses, for scanning\n" \
        "    \texamples: 208.67.1.1-254 or 2001:0DB8::1428:57ab-6344\n" \
        "  -p\tport (default 80)\n" \
        "Extra options:\n" \
        "  -t\ttime in seconds (default 3)\n" \
        "  -P\tpersonalized http request. Payload file\n" \
        "  -v\tfull answer (include html data)\n" \
        "  -j\tcookie jar separator (\"%%\")\n";

    fputs(usage, out);

    if(error)
        fputs(error, out);

    exit(1);
}

/*
 * Regular expression matching function
 * Returns:
 *    -1  error on pattern compilation
 *     0  don't match
 *     1  match
 */
int regmatch(const char* str, char* regex) {
    regex_t re;

    // Compile the regex pattern.
    if (regcomp(&re, regex, REG_EXTENDED | REG_NOSUB) != 0) {
        fprintf(stderr, "Error on regex pattern compilation.\n");
        return -1;
    }

    // Use the pattern on the passed string
    if (regexec(&re, str, 0, NULL, 0))  {
        return 0;
    }

    regfree(&re);
    return 1;
}

/*
 * Read http request from a file
 */
int readhttprequest(char *path) {
    FILE *file;
    char buffer[1024];

    memset(http_request, 0, MAX_DATA);

    if((file=fopen(path, "r"))==NULL)
        usage(stderr, "\nError opening file for read the http request.\n");

    if(!fread(http_request, 1, MAX_DATA, file))
        usage(stderr, "\nError reading http request from file.\n");

    fclose(file);
    
    return 1;
}

/*
 * Fill ip range struct and http_request with arguments
 * received from parseopt();
 * (perhaps you could find a more elegant way...)
 */
int parseipr(struct iprange *ipr, char *range, int port, int time, char *request, int verbose, int cookiejar)
{
    char *c, *x;


    return 1;
}

/*
 * Parser for command line options
 * See getopt(3)...
 */
int parseopt(struct iprange *ipr, int argc, char **argv)
{
    char *c, *x;
    int i=0;
    char ch;
    
    char range[MIN_ARG],  // ip range
         *request=NULL;   // http request
    
    int start,          // fist host
        end,            // last host
        curr,           // current host
        port=80,        // port
        time=3,         // child lifetime
        verbose,        // verbose
        cookiejar;      // cookie-jar output
    
    int ipver=0;   // ip version
    int single=0;  // single ip?
    
    
    // cleaning
    range[0]='\0';
    memset(ipr, 0, sizeof(struct iprange));
    
    
    char *shortopt = "r:p:t:P:vj"; // short options
    
    while ((ch = getopt (argc, argv, shortopt)) != -1)
        switch (ch) {
        
            case 'r': // IP range
                
                // ipv4 range
                if(regmatch(optarg, reg_ipv4_range)) {
                    ipver=VER_IPV4;
                }
                
                // ipv4 single
                else if(regmatch(optarg, reg_ipv4_addr)) {
                    ipver=VER_IPV4;
                    single=1;
                }
                
                // ipv6 range
                else if(regmatch(optarg, reg_ipv6_range)) {
                    ipver=VER_IPV6;
                }
                
                // ipv6 single
                else if(regmatch(optarg, reg_ipv6_addr)) {
                    ipver=VER_IPV6;
                    single=1;
                }
                
                else {
                    usage(stderr, "\nArgument range is invalid.\n");
                }
                
                strncpy(range, optarg, MIN_ARG);

                break;
            
            case 'p': // Port
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument port is invalid.\n");
                port = atoi(optarg);
                break;
            
            case 't': // Time
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument time is invalid.\n");
                time = atoi(optarg);
                break;
            
            case 'P': // Payload file
                request = optarg;
                break;
                
            case 'v': // Full answer
                verbose = 1;
                break;
            
            case 'j': // Cookie jar
                cookiejar = 1;
                break;
                            
            case '?':
            default:
                usage(stderr, NULL);
        }


    /* Check collected args */
    if(range[0]=='\0') {
        usage(stderr, "\nWhere is the range? Uff...\n");
    }
    
    if(time<1) time = 3;
    
    if(port<1) port=80;
    
    
    /* Fill iprange structure */
    
    // port
    ipr->port=port;
    
    // time
    ipr->time=time;    
    
    // verbose
    ipr->verbose=verbose;
    
    // cookiejar
    ipr->cookiejar=cookiejar;
    
    // ipversion
    ipr->ipver=ipver;

    // range
	if(range[0]!='\0') {
	
        strncpy(ipr->range, range, MIN_ARG);
        
        // ipv4
        if(ipver==VER_IPV4) {
            c = range;
            x = range;

            while(*c!='\0') c++;
            while(*c!='.') c--;

            *c = '\0';
            strncpy(ipr->block, range, MIN_ARG);
            *c = '.';

            // start
            if(!single) {
                while(*x!='-') x++;
                *x='\0';
            }
            
            c++;
            
            ipr->start=atoi(c);
    
            // single ip
            if(single) {
                strcat(ipr->range,"-");
                strcat(ipr->range,c);
                ipr->end=ipr->start;
            }
            
            // range
            else {
                x++;
                ipr->end=atoi(x);
            }
        }
        
        // ipv6
        else if(ipver==VER_IPV6) {
            c = range;
            x = range;

            while(*c!='\0') c++;
            while(*c!=':') c--;

            *c = '\0';
            strncpy(ipr->block, range, MIN_ARG);
            *c = ':';

            // start
            if(!single) {
                while(*x!='-') x++;
                *x='\0';
            }
            
            c++;
            
            sscanf(c, "%x", &ipr->start);

            // single ip
            if(single) {
                strcat(ipr->range,"-");
                strcat(ipr->range,c);
                ipr->end=ipr->start;
            }
            
            // range
            else {
                x++;
                sscanf(x, "%x", &ipr->end);
            }
        }
        
        // current
        ipr->curr = ipr->start;
	}
   
    // check range
    if(ipr->end < ipr->start)
        usage(stderr, "\nArgument ip-range is impossible ;)\n");
    
    //http_request
    if(request) readhttprequest(request);
    
    return 1;
}

#endif /* __HTTPARSER_H__ */

